#!/usr/bin/env python

from os import path
import yaml
from util import merge, get, log
from device import DeviceManager
from device_os import OS
from test import TestManager
from bds import BuildManager
from error import CapyException


################################
# Setup
################################
class Config:
    INCLUDE = 'include'

    def __init__(self, file_name, private_file_name=None):
        self.data = self.load_config(file_name, private_file_name)

        self.build_manager = BuildManager(get(self.data, 'bds', None), OS.all)
        self.device_manager = DeviceManager(get(self.data, 'devices', None), OS.all)
        self.test_manager = TestManager(get(self.data, 'tests', None))

    def load_yaml(self, file_name, check):
        if file_name is None:
            if check:
                raise CapyException("Missing configuration file.")
            else:
                return None

        # support both .yaml and .yml
        file_yaml = "%s.yaml" % file_name
        file_yml = "%s.yml" % file_name
        found_file = None

        if path.exists(file_yaml):
            found_file = file_yaml
        elif path.exists(file_yml):
            found_file = file_yml

        if found_file is None:
            if check:
                raise CapyException("Current directory does not contain configuration file '%s'. Please create one and run again." % file_yaml)
            else:
                return None

        with open(found_file, 'r') as stream:
            try:
                return yaml.load(stream)
            except yaml.YAMLError as ex:
                raise CapyException(ex.message)

    def load_config(self, file_name, private_file_name):
        data = self.load_yaml(file_name, check=True)
        data = self.apply_includes(data)
        private_data = self.load_yaml(private_file_name, check=False)
        private_data = self.apply_includes(private_data)

        if private_data is not None:
            private_data = merge(private_data, data)
            return private_data
        else:
            log.verbose("Private config file '%s.yaml' not found." % private_file_name)
            return data

    def apply_includes(self, data):
        result = {}

        if data is not None:
            for key, value in data.iteritems():
                if value is None:
                    raise CapyException("'%s:' is empty. Assign it a value!" % key)

                if self.INCLUDE in value and value[self.INCLUDE] is not None:
                    included_value = self.load_yaml(value[self.INCLUDE], True)
                    result[key] = merge(included_value, value)
                else:
                    result[key] = value

        return result




#!/usr/bin/env python

import sys
import os
import time
from os import path, makedirs
import subprocess
import json
from device_os import OS
from util import Color, TMP_DIR, get, log
from error import CapyException


################################
# Build Manager
#
# cli command: curl -O $(curl -u 'token':'' -s http://inloop-bds.test.inloop.eu/api/v1/customers/medrio/projects/mcapture/applications/ios/environments/internal-calabash/builds/ | python -c 'import sys, json; print json.load(sys.stdin)["builds"][0]["download_url"]')
#
################################
class BuildManager(object):
    API_ENDPOINT = 'https://inloop-bds.inloop.eu/api/v1'

    def __init__(self, conf, os_list):
        if not conf:
            raise CapyException('BDS configuration is missing')

        conf['build_dir'] = conf.get('build_dir', path.join(TMP_DIR + 'builds/'))
        self.token = get(conf, 'token', None)  # don't check token until it's needed
        self.customer = self._load(conf, 'customer')
        self.project = self._load(conf, 'project')

        self.builds = {}
        for os in os_list:
            self._load_builds(conf, os=os)

    def get_token(self):
        if not self.token:
            raise CapyException("BDS configuration is missing a 'token'")
        return self.token

    def download(self, build):
        # load build from BDS
        bds_build = self._get_latest_bds_build(build)
        download_url = bds_build['download_url']
        log.raw(Color.BLUE + 'Downloading from url %s...' % download_url + Color.ENDC)
        # download
        download_to = build.get_path()

        if path.exists(download_to):
            log.raw(Color.BLUE + 'Removing previous %s...' % download_to + Color.ENDC)
            subprocess.call(['rm', download_to])

        # execute download
        download_proc = subprocess.Popen(
            ['curl', '-o', download_to, download_url], stdout=sys.stdout, stderr=sys.stderr
        )
        download_proc.wait()
        download_proc.communicate()

        if path.exists(download_to):
            log.raw(Color.BLUE + 'Downloaded to ' + download_to + Color.ENDC)
            if build.os == OS.Android:
                # resign build for Android
                log.raw(Color.BLUE + 'Resigning apk...' + Color.ENDC)
                subprocess.call(['bundle', 'exec', 'calabash-android', 'resign', download_to])
        else:
            raise CapyException('BDS build could not be downloaded')

    def check_and_get_build(self, os, build_name):
        build = self.get_build(os, build_name)

        build_path = build.get_path()
        if not path.exists(build_path):
            self.download(build)

        return build

    def get_build(self, os, build_name):
        if build_name:
            build = self.get_builds(os).get(build_name, None)
        else:
            build = self._get_default_build(os)

        if build:
            return build
        else:
            raise CapyException("Build with name '%s' does not exists for '%s'!" % (build_name, os))

    def get_builds(self, os):
        builds = self.builds.get(os, None)
        if builds:
            return builds
        else:
            raise CapyException("No %s builds were found!" % os)

    def get_version_names(self, build):
        url = self._prepare_url(
            os=build.os,
            env=build.env,
            conf=build.conf,
            end='versions/'
        )

        response = self._download_json(url)

        def parse_version(obj):
            return str(obj['version'])

        versions = map(parse_version, response['versions'])

        return versions

    def _get_default_build(self, os):
        for name, build in self.get_builds(os).iteritems():
            if build.is_default:
                return build

        raise CapyException("'%s' has no default build! Please add 'default: true' to one of the builds." % os)

    def _load(self, conf, prop):
        p = conf.get(prop, None)
        if not p:
            raise CapyException("BDS configuration is missing a '%s'" % prop)
        return p

    def _load_builds(self, conf, os):
        builds = {}

        os_conf = get(conf, os, None)
        if not os_conf:
            return

        default_build_found = False
        default_build_name = get(os_conf, 'default', None)
        if not default_build_name:
            raise CapyException("BDS is missing default build for '%s'" % os)

        for name, info in os_conf.iteritems():
            if name == 'default':
                continue

            info['build_dir'] = info.get('build_dir', conf['build_dir'])
            build = Build(os, name, info)
            self._validate_version(build)

            if name == default_build_name:
                build.is_default = True
                default_build_found = True
            builds[name] = build

        if not default_build_found:
            raise CapyException("'%s' default build '%s' was not found" % (os, default_build_name))

        self.builds[os] = builds

    def _validate_version(self, build):
        if not build.version:
            return # do nothing if not specified

        versions = self.get_version_names(build)
        if build.version not in versions:
            raise CapyException("'{os}' build '{name}' has invalid version name '{version}'.\nSupported versions are {versions}".format(
                os=build.os, name=build.name, version=build.version, versions=versions
            ))

    def _get_latest_bds_build(self, build):
        url = self._prepare_url(
            os=build.os,
            env=build.env,
            conf=build.conf,
            version=build.version,
            end='builds/'
        )

        try:
            response = self._download_json(url)
            return response['builds'][0]  # get 1st build
        except:
            raise CapyException('No BDS build was found')

    def _download_json(self, url):
        token = "%s:\'\'" % self.get_token()
        cmd = ['curl', '-u', token, '-s', url]
        c = ' '.join(cmd)

        temp_file_path = 'temp_download_%s.json' % time.time()

        try:
            # write cmd output to a temporary file (because when piping to the main process the subprocess hangs)
            with open(temp_file_path, 'w+') as temp_file:
                proc = subprocess.Popen(c, shell=True, stdout=temp_file)
            proc.wait()

            # read response json
            with open(temp_file_path, 'r') as temp_file:
                response_json = json.load(temp_file)

            # check if we have a response
            if response_json is None:
                raise CapyException('Unauthorized - Your BDS token expired!')

            # clear temp file
            os.remove(temp_file_path)

            return response_json
        except:
            # clear temp file
            if os.path.exists(temp_file_path):
                os.remove(temp_file_path)

            raise CapyException('JSON could not be downloaded from ' + url)

    def _prepare_url(self, os, env=None, conf=None, version=None, end=None):
        # mandatory url parts:
        url = '/customers/{customer}/projects/{project}/applications/{os}/'.format(
            customer=self.customer, project=self.project, os=os
        )
        # optional url parts:
        if env:
            url += 'environments/{env}/'.format(env=env)
        if conf:
            url += 'configurations/{conf}/'.format(conf=conf)
        if version:
            url += 'versions/{version}/'.format(version=version)
        if end:
            url += end

        return self.API_ENDPOINT + url


class Build(object):
    def __init__(self, os, name, info):
        self.os = os
        self.name = name
        self.is_default = False
        self.app_id = info.get('app_id', None)
        if not self.app_id:
            raise CapyException("BDS Build '%s' must specify an 'app_id'" % self.name)
        self.csid = info.get('csid', None)
        if self.csid is not None:
            self.csid = self.csid.encode('utf-8')
        if os == OS.iOS and not self.csid:
            raise CapyException("BDS iOS Build '%s' must specify a 'csid' (Code Sign Identity)" % self.name)
        self.env = info.get('env', None)
        self.conf = info.get('conf', None)
        self.version = info.get('version', None)
        self.build_dir = info['build_dir']
        # prepare path
        extension = '.apk' if os == OS.Android else '.ipa'
        self.file_name = name + extension

    def get_path(self):
        build_path = path.join(self.build_dir, self.os)
        if not path.exists(build_path):
            makedirs(build_path)
        return path.join(build_path, self.file_name)

    def show(self, line_start=''):
        if self.is_default:
            s = line_start + Color.LIGHT_RED + self.name + Color.RED + ' (default)'
        else:
            s = line_start + Color.LIGHT_GREEN + self.name

        s += '\n' + line_start + Color.YELLOW + '  - app ID: ' + Color.ENDC + self.app_id
        if self.env:
            s += '\n' + line_start + Color.YELLOW + '  - env: ' + Color.ENDC + self.env
        if self.conf:
            s += '\n' + line_start + Color.YELLOW + '  - conf: ' + Color.ENDC + self.conf
        if self.version:
            s += '\n' + line_start + Color.YELLOW + '  - version: ' + Color.ENDC + self.version
        if self.csid:
            s += '\n' + line_start + Color.YELLOW + '  - csid: ' + Color.ENDC + self.csid
        return s



#!/usr/bin/env python

import sys
from os import path
from util import Color, get, merge, TMP_DIR
from error import CapyException


################################
# Test Manager
################################
class TestManager(object):
    def __init__(self, conf):
        if not conf:
            conf = {}

        conf['output_dir'] = get(conf, 'output_dir', path.join(TMP_DIR))
        conf['env'] = get(conf, 'env', {})
        conf['before'] = get(conf, 'before', [])
        TestAction.validate(conf['before'])
        conf['after'] = get(conf, 'after', [])
        TestAction.validate(conf['after'])

        self.tests = self.load_tests(conf)

    def load_tests(self, conf):
        tests = {}

        for name, info in conf.iteritems():
            if name in ['output_dir', 'env', 'before', 'after']:
                continue

            info = merge(info, conf)
            tests[name] = Test(name, info)

        return tests

    def get_test(self, name):
        test = self.tests.get(name, None)
        if test:
            return test
        else:
            raise CapyException("Test '%s' was not found" % name)


################################################################
# Test
#
# NOTE: Scenarios won't necessarily run in order of given tags
# (Calabash goes through all feature files and their scenarios
# a executes those scenarios that match the tags)
################################################################
class Test:
    def __init__(self, name, conf):
        self.name = name
        self.output_dir = conf['output_dir']
        self.env = conf['env']

        self.run = get(conf, 'run', None)
        if not self.run:
            raise CapyException("Test '%s' is missing a 'run: ...'" % name)

        self.before = get(conf, 'before', [])
        TestAction.validate(self.before)
        self.after = get(conf, 'after', [])
        TestAction.validate(self.after)

    def get_env(self):
        return self.env

    def show(self, line_start=''):
        s = line_start + Color.LIGHT_GREEN + self.name + ":\n"
        # show run
        s += line_start + '  ' + self.run + Color.ENDC
        s = s.replace('@', Color.LIGHT_RED + '@' + Color.ENDC)
        s = s.replace('--tags', Color.YELLOW + '--tags')
        s = s.replace(',', Color.YELLOW + ',')
        # show actions
        if self.before:
            s += '\n'
            s += line_start + Color.YELLOW + '  before: ' + Color.ENDC + ', '.join(self.before) + Color.ENDC
        if self.after:
            s += '\n'
            s += line_start + Color.YELLOW + '  after: ' + Color.ENDC + ', '.join(self.after)
        # show ENV
        if self.env:
            s += '\n'
            s += line_start + Color.YELLOW + '  env: ' + Color.ENDC + str(self.env)

        return s


################################################################
# Action that can be run before or after a test
################################################################
class TestAction:
    DOWNLOAD = 'download'
    INSTALL = 'install'
    UNINSTALL = 'uninstall'
    ALL = [DOWNLOAD, INSTALL, UNINSTALL]

    @classmethod
    def validate(cls, actions):
        for action in actions:
            if action not in cls.ALL:
                raise CapyException("Test action '%s' is not supported" % action)



#!/usr/bin/env python

from util import Color, get
from error import CapyException
from device_os import OS
import os


################################
# Device Manager
################################
class DeviceManager(object):
    def __init__(self, conf, os_list):
        if not conf:
            conf = {}

        self.devices = {}
        for os in os_list:
            self.load_devices(conf, os=os)

    # private
    def load_devices(self, conf, os):
        for name, info in conf.get(os, {}).iteritems():
            if os == OS.Android:
                id = get(info, 'id', None)
                port = get(info, 'port', None)
                if id and not port:
                    raise CapyException("Device '%s' is missing parameter 'port' (it is required if 'id' is specified)" % name)

                self.devices[name] = AndroidDevice(name, id, port)
            elif os == OS.iOS:
                # change uuid to id (to ensure compatibility with older versions)
                if 'uuid' in info:
                    info['id'] = info['uuid']
                    del info['uuid']

                self.validate_device(name, info, 'id')
                self.validate_device(name, info, 'ip')

                self.devices[name] = IosDevice(name, info['id'], info['ip'])

    # private
    def validate_device(self, name, params, param_name):
        if param_name not in params.keys():
            raise CapyException("Device '%s' is missing parameter '%s'" % (name, param_name))

    # public
    def get_device(self, name):
        device = self.devices.get(name, None)
        if device:
            return device
        else:
            raise CapyException("Device '%s' was not found" % name)


################################
# Base Device
################################
class BaseDevice(object):
    DEVICE_NAME = 'CAPY_DEVICE_NAME'

    def __init__(self, os, name, env={}):
        self.os = os
        self.name = name
        self.env = env
        self.env[self.DEVICE_NAME] = name

    def get_os(self):
        return self.os

    def get_env(self):
        return self.env

    def get_cli_tools(self):
        return []  # implement

    def get_install_cmds(self, build):
        return []  # implement

    def get_uninstall_cmds(self, build):
        return []  # implement

    def get_console_cmd(self, build):
        return []  # implement

    def get_run_cmd(self, build):
        return []  # implement

    def get_build_env(self, build):
        return {}  # implement

    def show(self, line_start=''):
        return line_start + Color.LIGHT_GREEN + '%s ' % self.name + Color.YELLOW + '(%s)' % self.os + Color.ENDC


################################
# iOS Device
################################
class IosDevice(BaseDevice):
    CLI_TOOL = 'ideviceinstaller'
    ID_ENV_NAME = 'DEVICE_TARGET'
    IP_ENV_NAME = 'DEVICE_ENDPOINT'

    def __init__(self, name, id, ip):
        env = {
            self.ID_ENV_NAME: id,
            self.IP_ENV_NAME: 'http://%s:37265' % ip
        }
        super(IosDevice, self).__init__(OS.iOS, name, env)

    def get_console_cmd(self, build):
        return ['bundle', 'exec', 'calabash-ios', 'console', '-p', 'ios']

    def get_run_cmd(self, build):
        return ['bundle', 'exec', 'cucumber', '-p', 'ios']

    def get_build_env(self, build):
        return {
            "BUNDLE_ID": build.app_id,
            "CODE_SIGN_IDENTITY": build.csid
        }

    def get_cli_tools(self):
        return [self.CLI_TOOL]

    def get_install_cmds(self, build):
        return [
            [self.CLI_TOOL, '-u', self.env[self.ID_ENV_NAME], '-i', build.get_path()]
        ]

    def get_uninstall_cmds(self, build):
        cmds = [
            [self.CLI_TOOL, '-u', self.env[self.ID_ENV_NAME], '-U', build.app_id]
        ]
        if build.csid:
            cmds.append([self.CLI_TOOL, '-u', self.env[self.ID_ENV_NAME], '-U', 'com.apple.test.DeviceAgent-Runner'])

        return cmds

    def show(self, line_start=''):
        s = super(IosDevice, self).show(line_start=line_start)
        s += '\n' + line_start + Color.YELLOW
        s += '  - ID: ' + Color.ENDC + '%s' % self.env[self.ID_ENV_NAME]
        s += Color.ENDC
        s += '\n' + line_start + Color.YELLOW
        s += '  - IP: ' + Color.ENDC + '%s' % self.env[self.IP_ENV_NAME]
        s += Color.ENDC
        return s


################################
# Android Device
################################
class AndroidDevice(BaseDevice):
    CLI_TOOL = 'adb'
    ID_ENV_NAME = 'ADB_DEVICE_ARG'
    PORT_ENV_NAME = 'TEST_SERVER_PORT'

    def __init__(self, name, id=None, port=None):
        env = {}
        if id:
            env[self.ID_ENV_NAME] = str(id)
        if port:
            env[self.PORT_ENV_NAME] = str(port)  # make sure it's a string
        super(AndroidDevice, self).__init__(OS.Android, name, env)

    def get_console_cmd(self, build):
        return ['bundle', 'exec', 'calabash-android', 'console', build.get_path(), '-p', 'android']

    def get_run_cmd(self, build):
        return ['bundle', 'exec', 'calabash-android', 'run', build.get_path(), '-p', 'android']

    def get_cli_tools(self):
        return [self.CLI_TOOL]

    def get_install_cmds(self, build):
        cmds = []

        # rebuild test-server
        cmds.append(['bundle', 'exec', 'calabash-android', 'build', build.get_path()])

        # install app
        install_cmd = [self.CLI_TOOL]
        if self.PORT_ENV_NAME in self.env:
            install_cmd.append('-s')
            install_cmd.append(self.env[self.ID_ENV_NAME])
        install_cmd.append('install')

        android_version = self._get_device_android_version()
        if android_version < 23:
            install_cmd.append('-r')
        else:
            # since 6.0 (API 23) you need '-g' in order to grant runtime permissions
            install_cmd.append('-g')
            install_cmd.append('-r')

        install_cmd.append(build.get_path())
        cmds.append(install_cmd)

        return cmds

    def _get_device_android_version(self):
        if self.PORT_ENV_NAME in self.env:
            cmd = "{a} -s {n} shell getprop ro.build.version.sdk".format(a=self.CLI_TOOL, n=self.env[self.ID_ENV_NAME])
        else:
            cmd = "{a} shell getprop ro.build.version.sdk".format(a=self.CLI_TOOL)

        version_str = os.popen(cmd).read()
        return int(version_str)

    def get_uninstall_cmds(self, build):
        if self.PORT_ENV_NAME in self.env:
            return [
                [self.CLI_TOOL, '-s', self.env[self.ID_ENV_NAME], 'uninstall', build.app_id]
            ]
        else:
            return [
                [self.CLI_TOOL, 'uninstall', build.app_id]
            ]

    def show(self, line_start=''):
        s = super(AndroidDevice, self).show(line_start=line_start)
        if self.ID_ENV_NAME in self.env:
            s += '\n' + line_start + Color.YELLOW
            s += '  - ID: ' + Color.ENDC + '%s' % self.env[self.ID_ENV_NAME]
            s += Color.ENDC
        if self.PORT_ENV_NAME in self.env:
            s += '\n' + line_start + Color.YELLOW
            s += '  - PORT: ' + Color.ENDC + '%s' % self.env[self.PORT_ENV_NAME]
            s += Color.ENDC
        return s



#!/usr/bin/env python

import sys
import argparse
import urllib, json
from datetime import datetime
from conf import Config
from util import Color, STDERR_LOG_MANAGER, STDOUT_LOG_MANAGER, check_cmd, log
from test import TestAction
from cmd import DeviceRunner
from error import CapyException

DESCRIPTION = '''CAPY is a helper for running calabash tests on iOS and Android'''
LONG_DESCRIPTION = DESCRIPTION
NAME = 'capy'
VERSION = '1.1.12'


####################################################################################################
# Version check
####################################################################################################
def check_version():
    msg = check_package(NAME, VERSION)
    if msg:
        c = Color.LIGHT_GREEN
        log.raw(c + '+----------------------------------------+')
        log.raw(c + '| {m:30}'.format(m=msg) + c + ' |')
        log.raw(c + '| {m:38}'.format(m=' ') + c + ' |')
        log.raw(c + '| {m:42}'.format(m='Please run: ' + Color.ENDC + 'pip install -U ' + NAME) + c + ' |')
        log.raw(c + '+----------------------------------------+' + Color.ENDC)


def check_package(name, current_version):
    try:
        response = urllib.urlopen('https://pypi.python.org/pypi/%s/json' % name)
        data = json.loads(response.read())
        latest_version = data['info']['version']
        if latest_version != current_version:
            return '%s has new release (%s) available' % (name, latest_version)
        else:
            return None
    except IOError:
        return None  # show nothing if request failed


def check_calabash():
    cmds = ['calabash-android', 'cucumber']
    for cmd in cmds:
        if not check_cmd(cmd):
            raise CapyException('Command %s was NOT found. Please make sure calabash is installed.' % cmd)


####################################################################################################
# Helper methods
####################################################################################################
def get_config():
    return Config(file_name='capy_conf', private_file_name='capy_private')


def read_build(args):
    return args.build[0] if args.build else None


def show_info():
    log.raw('%s %s' % (NAME, VERSION))
    log.raw(DESCRIPTION)


def console(build_name, device_name):
    check_calabash()

    config = get_config()
    device = config.device_manager.get_device(device_name)
    build = config.build_manager.check_and_get_build(device.os, build_name)
    log.raw(Color.GREEN + "Opening console for device '%s' with '%s'..." % (device.name, build.name) + Color.ENDC)
    DeviceRunner(device).open_console(build)


def run(build_name, device_name, test_name, with_report=False):
    check_calabash()

    # save execution start
    start_time = datetime.now().replace(microsecond=0)

    config = get_config()

    device = config.device_manager.get_device(device_name)
    build = config.build_manager.get_build(device.os, build_name)
    test = config.test_manager.get_test(test_name)

    if with_report:
        # use custom loggers
        sys.stdout = STDOUT_LOG_MANAGER
        STDOUT_LOG_MANAGER.start_for_device(device)
        sys.stderr = STDERR_LOG_MANAGER
        STDERR_LOG_MANAGER.start_for_device(device)

    if test.before:
        for action in test.before:
            exec_action(action, config, build, device)

    # just to make sure build is available (this will download it if not)
    build = config.build_manager.check_and_get_build(device.os, build_name)
    version_names_getter = lambda: config.build_manager.get_version_names(build)

    log.raw(Color.GREEN + "Running '%s' on device '%s' with '%s'..." % (test.name, device.name, build.name) + Color.ENDC)
    runner = DeviceRunner(device)
    runner.run_test(test, build, version_names_getter, report=with_report)

    if test.after:
        for action in test.after:
            exec_action(action, config, build, device)

    # show time
    end_time = datetime.now().replace(microsecond=0)
    diff = end_time - start_time
    log.raw('+-------------------------------------------------------------------------')
    log.raw('| Total testing time is: ' + diff)
    log.raw('+-------------------------------------------------------------------------')

    if with_report and runner.latest_report_dir:
        # move logs
        STDOUT_LOG_MANAGER.move_to(runner.latest_report_dir)
        STDERR_LOG_MANAGER.stop()
        STDERR_LOG_MANAGER.move_to(runner.latest_report_dir)
        STDERR_LOG_MANAGER.stop()


def exec_action(test_action, config, build, device):
    log.raw(Color.GREEN + "Running action '%s' on device '%s' with '%s'..." % (test_action, device.name, build.name) + Color.ENDC)
    if test_action == TestAction.DOWNLOAD:
        config.build_manager.download(build)
    elif test_action == TestAction.INSTALL:
        DeviceRunner(device).install(build)
    elif test_action == TestAction.UNINSTALL:
        DeviceRunner(device).uninstall(build)


def list(builds=False, devices=False, tests=False):
    config = get_config()

    line_start = Color.GREEN

    log.raw(line_start + '+------------------------------------------------------------------------------------' + Color.ENDC)
    if builds:
        log.raw(line_start + '| ' + Color.LIGHT_YELLOW + 'BUILDS:')
        log.raw(line_start + '|')
        for os, builds_dict in config.build_manager.builds.iteritems():
            log.raw(line_start + '| ' + os)
            for name, build in sorted(builds_dict.iteritems()):
                log.raw(build.show(line_start + '|    '))
        log.raw(line_start + '|------------------------------------------------------------------------------------' + Color.ENDC)
    if devices:
        log.raw(line_start + '| ' + Color.LIGHT_YELLOW + 'DEVICES:')
        log.raw(line_start + '|')
        for name, device in sorted(config.device_manager.devices.iteritems()):
            log.raw(device.show(line_start + '| '))
        log.raw(line_start + '|------------------------------------------------------------------------------------' + Color.ENDC)
    if tests:
        log.raw(line_start + '| ' + Color.LIGHT_YELLOW + 'TESTS:')
        log.raw(line_start + '|')
        for name, test in sorted(config.test_manager.tests.iteritems()):
            log.raw(test.show(line_start + '| '))
        log.raw(line_start + '+------------------------------------------------------------------------------------' + Color.ENDC)


def download(build_name, os):
    check_calabash()

    config = get_config()
    build = config.build_manager.get_build(os, build_name)
    log.raw(Color.GREEN + "Downloading build '%s' for '%s'..." % (build.name, build.os) + Color.ENDC)
    config.build_manager.download(build)


def install(build_name, device_name):
    check_calabash()

    config = get_config()
    device = config.device_manager.get_device(device_name)
    build = config.build_manager.check_and_get_build(device.os, build_name)
    log.raw(Color.GREEN + "Installing '%s' to device '%s'..." % (build.name, device.name) + Color.ENDC)
    DeviceRunner(device).install(build)


def uninstall(build_name, device_name):
    check_calabash()

    config = get_config()
    device = config.device_manager.get_device(device_name)
    build = config.build_manager.check_and_get_build(device.os, build_name)
    log.raw(Color.GREEN + "Uninstalling '%s' from device '%s'..." % (build.name, device.name) + Color.ENDC)
    DeviceRunner(device).uninstall(build)


###########################################################
# Main
###########################################################
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-b', '--build', nargs=1, metavar='B',
                        help="Choose different build B to use for tests")
    parser.add_argument('-c', '--console', nargs=1, metavar='D',
                        help="Open calabash console for device D")
    parser.add_argument('-d', '--download', choices=['android', 'ios'],
                        help="Download build for given platform")
    parser.add_argument('-i', '--install', nargs=1, metavar='D',
                        help="Install current build on device D")
    parser.add_argument('-l', '--list', action='store_true',
                        help="List all supported builds, devices and tests")
    parser.add_argument('-lb', '--list-build', action='store_true',
                        help="List all supported builds")
    parser.add_argument('-ld', '--list-device', action='store_true',
                        help="List all supported devices")
    parser.add_argument('-lt', '--list-test', action='store_true',
                        help="List all supported tests")
    parser.add_argument('-r', '--run', nargs=2, metavar=('D', 'T'),
                        help="Run test T on device D")
    parser.add_argument('-rr', '--run-report', nargs=2, metavar=('D', 'T'),
                        help="Run test T on device D and create HTML report")
    parser.add_argument('-v', '--verbose', action='store_true',
                        help="Use verbose mode")
    parser.add_argument('--version', action='store_true',
                        help="Show version info")
    parser.add_argument('-u', '--uninstall', nargs=1, metavar='D',
                        help="Uninstall build from device D")
    args = parser.parse_args()
    try:
        main_run(parser, args)
    except CapyException as ex:
        log.error(ex.message)
        sys.exit(1)


def main_run(parser, args):
    log.setup_verbosity(args.verbose is True)

    # run
    if args.run:
        run(build_name=read_build(args), device_name=args.run[0], test_name=args.run[1])
    elif args.run_report:
        run(build_name=read_build(args), device_name=args.run_report[0], test_name=args.run_report[1], with_report=True)
    # console
    elif args.console:
        console(build_name=read_build(args), device_name=args.console[0])
    # list
    elif args.list:
        list(builds=True, devices=True, tests=True)
    elif args.list_build:
        list(builds=True)
    elif args.list_device:
        list(devices=True)
    elif args.list_test:
        list(tests=True)
    # version
    elif args.version:
        show_info()
    # download
    elif args.download:
        download(build_name=read_build(args), os=args.download)
    # install
    elif args.install:
        install(build_name=read_build(args), device_name=args.install[0])
    # uninstall
    elif args.uninstall:
        uninstall(build_name=read_build(args), device_name=args.uninstall[0])
    # show help by default
    else:
        parser.parse_args(['--help'])

    # check for updates
    check_version()


################################
# run main
################################
if __name__ == '__main__':
    main()



#!/usr/bin/env python

import re
import sys
import subprocess
import shutil
from os import makedirs, path, rename
from error import CapyException

####################################################################################################
# Temporary directory for all files
####################################################################################################
TMP_DIR = '.capy/'


####################################################################################################
# Dictionary util functions
####################################################################################################
def merge(user, default):
    if isinstance(user, dict) and isinstance(default, dict):
        for k, v in default.iteritems():
            if k not in user:
                user[k] = v
            else:
                user[k] = merge(user[k], v)
    return user


def get(conf, prop, default):
    if not conf:
        return default

    p = conf.get(prop, None)
    if p:
        return p
    else:
        return default


####################################################################################################
# Console colors
####################################################################################################
class Color:
    ENDC = '\033[0m'

    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    DEFAULT = '\033[39m'
    BLACK = '\033[30m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    MANGETA = '\033[35m'
    CYAN = '\033[36m'
    LIGHT_GRAY = '\033[37m'
    DARK_GRAY = '\033[90m'
    LIGHT_RED = '\033[91m'
    LIGHT_GREEN = '\033[92m'
    LIGHT_YELLOW = '\033[93m'
    LIGHT_BLUE = '\033[94m'
    LIGHT_MANGETA = '\033[95m'
    LIGHT_CYAN = '\033[96m'
    WHITE = '\033[97m'


####################################################################################################
# Logs console messages
####################################################################################################
class PrintLog(object):
    def __init__(self):
        self.verbose = self._verbose_noop

    def error(self, msg):
        print Color.LIGHT_RED + 'Error: %s' % msg + Color.ENDC

    def warning(self, msg):
        print Color.LIGHT_YELLOW + 'Warning: %s' % msg + Color.ENDC

    def raw(self, msg):
        print msg

    def _verbose_real(self, msg):
        print Color.LIGHT_CYAN + 'Info: %s' % msg + Color.ENDC

    def _verbose_noop(self, msg):
        pass

    def setup_verbosity(self, verbose=False):
        if verbose:
            self.verbose = self._verbose_real
        else:
            self.verbose = self._verbose_noop


log = PrintLog()


####################################################################################################
# Custom logger
####################################################################################################
class LogOutputManager(object):
    def __init__(self, base_file_name, pipe=None):
        if not path.exists(TMP_DIR):
            makedirs(TMP_DIR)
        self.base_file_name = base_file_name
        self.pipe = pipe
        self.file_name = None
        self.file_path = None
        self.is_used = False

    def start_for_device(self, device):
        self.file_name = '{d}_{b}'.format(d=device.name, b=self.base_file_name)
        self.file_path = path.join(TMP_DIR, self.file_name)
        self.is_used = True
        # make sure log file exists and is empty
        with open(self.file_path, 'w') as log_file:
            log_file.write('')

    def stop(self):
        self.is_used = False

    def write(self, message):
        if self.pipe:
            self.pipe.write(message)
        colorless = re.sub(r"\[[0-9]{1,2}m", "", message)
        with open(self.file_path, "a") as log_file:
            log_file.write(colorless)

    def flush(self):
        # this flush method is needed for python 3 compatibility.
        # this handles the flush command by doing nothing.
        # you might want to specify some extra behavior here.
        pass

    def fileno(self):
        if self.pipe:
            self.pipe.fileno()

    def move_to(self, dst):
        shutil.move(self.file_path, dst)
        rename(path.join(dst, self.file_name), path.join(dst, self.base_file_name))


STDOUT_LOG_MANAGER = LogOutputManager('stdout.log', sys.stdout)
STDERR_LOG_MANAGER = LogOutputManager('stderr.log', sys.stderr)


####################################################################################################
# Check if command with given name is available
####################################################################################################
def check_cmd(name):
    proc = subprocess.Popen(['which', name], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    proc.wait()
    stdout, stderr = proc.communicate()
    return stdout and not stderr



from os import environ, makedirs, path
import subprocess
import time
from util import merge, TMP_DIR, STDERR_LOG_MANAGER, STDOUT_LOG_MANAGER, check_cmd, log
import shutil


################################
# Device Runner
################################
class DeviceRunner(object):
    def __init__(self, device):
        self.device = device
        self.latest_report_dir = None

    def check(self, name):
        if not check_cmd(name):
            self.call(['brew', 'install', name])

    # cmd is a string array
    def call(self, cmd, env=None):
        if STDOUT_LOG_MANAGER.is_used:
            # if custom logger is used, use it too
            main_proc = subprocess.Popen(cmd, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            tee_proc = subprocess.Popen(['tee', '-a', STDOUT_LOG_MANAGER.file_path], stdin=main_proc.stdout,
                                        stdout=STDOUT_LOG_MANAGER, stderr=STDERR_LOG_MANAGER)
            tee_proc.wait()
            tee_proc.communicate()
        else:
            subprocess.call(cmd, env=env)

    def prepare_env(self, *envs):
        base = environ.copy()
        for env in envs:
            base = merge(base, env)
        return base

    def install(self, build):
        self.inner_install(build, True)

    def uninstall(self, build):
        self.inner_install(build, False)

    def inner_install(self, build, install=True):
        for tool in self.device.get_cli_tools():
            self.check(tool)

        cmds = self.device.get_install_cmds(build) if install else self.device.get_uninstall_cmds(build)

        env = self.prepare_env(
                self.device.get_env(),
                self.device.get_build_env(build)
        )

        for cmd in cmds:
            self.call(cmd, env)

    def open_console(self, build):
        cmd = self.device.get_console_cmd(build)
        env = self.prepare_env(
                self.device.get_env(),
                self.device.get_build_env(build)
        )
        self.call(cmd, env)

    def run_test(self, test, build, version_names_getter, report=False):
        timestamp = time.strftime('%Y_%m_%d-%H_%M_%S')
        tmp = TMP_DIR
        tmp_out = self.get_current_report_dir(parent=tmp, timestamp=timestamp)
        real_out = self.get_device_reports_dir(parent=test.output_dir)

        # prepare command
        base_cmd = self.device.get_run_cmd(build)
        test_cmd = TestCmdBuilder.build_cmd(test, tmp_out, report)
        version_tags = VersionCmdBuilder.build_cmd(self.device, build, version_names_getter)
        cmd = base_cmd + test_cmd + version_tags

        # prepare env
        env = self.prepare_env(
                self.device.get_env(),
                self.device.get_build_env(build),
                test.get_env(),
                {
                    "SCREENSHOT_PATH": tmp_out + '/'  # has to end with '/'
                }
        )

        # show commands
        log.raw('--------------------------------------------------------------------------')
        log.raw('| Commands: ')
        log.raw('|')
        log.raw('| ' + ' '.join(cmd))
        log.raw('|')
        log.raw('| NOTE: output files will be moved to: ' + real_out)
        log.raw('|')
        log.raw('--------------------------------------------------------------------------')

        # run command
        self.call(cmd, env)

        # move files if necessary
        if tmp != test.output_dir:
            shutil.move(tmp_out, real_out)

        self.latest_report_dir = self.get_current_report_dir(parent=test.output_dir, timestamp=timestamp)

    def get_reports_dir(self, parent):
        dir = path.join(parent, 'reports/')
        if not path.exists(dir):
            makedirs(dir)
        return path.abspath(dir)

    def get_device_reports_dir(self, parent):
        dir = path.join(self.get_reports_dir(parent), '%s-%s/' % (self.device.os, self.device.name))
        if not path.exists(dir):
            makedirs(dir)
        return path.abspath(dir)

    def get_current_report_dir(self, parent, timestamp):
        dir = path.join(self.get_device_reports_dir(parent), timestamp)
        if not path.exists(dir):
            makedirs(dir)
        return dir


################################
# Version Command Builder
################################
class VersionCmdBuilder(object):
    MIN_PREFIX = '-min-v'
    MAX_PREFIX = '-max-v'

    @classmethod
    def build_cmd(cls, device, build, version_names_getter):
        if not build.version:
            return []

        sorted_versions = sorted(version_names_getter())
        index = sorted_versions.index(build.version)
        before = sorted_versions[:index]
        after = sorted_versions[index + 1:]

        # use negation via '~' in order to support tests without min/max tags
        min_cmd = ['~@' + device.get_os() + cls.MIN_PREFIX + tag for tag in after]
        max_cmd = ['~@' + device.get_os() + cls.MAX_PREFIX + tag for tag in before]

        cmd = []
        for tag in min_cmd + max_cmd:
            cmd += ['--tags', tag]

        return cmd


################################
# Test Command Builder
################################
class TestCmdBuilder(object):
    @classmethod
    def build_cmd(cls, test, output_dir_path, report):
        command = test.run.split(' ')

        if report:
            report_file = path.join(output_dir_path, 'report.html')
            command.append('--format')
            command.append('html')
            command.append('--out')
            command.append(report_file)
            command.append('--format')
            command.append('pretty')

        return command



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup
from capy import capy

setup(
    author='František Gažo',
    author_email='frantisek.gazo@inloop.eu',
    name=capy.NAME,
    version=capy.VERSION,
    description=capy.DESCRIPTION,
    long_description=capy.LONG_DESCRIPTION,
    url='https://github.com/FrantisekGazo/capy/',
    platforms=['MacOS'],
    license='MIT License',
    classifiers=[ # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Programming Language :: Python :: 2.7',
        'Development Status :: 1 - Planning',
        'Operating System :: MacOS',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Topic :: Utilities'
    ],
    packages=[
        'capy'
    ],
    install_requires=[ # list of this package dependencies
        'PyYAML>=3.11'
    ],
    entry_points='''
        [console_scripts]
        capy=capy.capy:main
    '''
)